@@ -40,6 +40,7 @@
 #include <spa/utils/string.h>
 #include <spa/utils/json.h>
 #include <spa/utils/ringbuffer.h>
+#include <spa/utils/dll.h>
 #include <spa/debug/pod.h>
 #include <spa/pod/builder.h>
 #include <spa/param/audio/format-utils.h>
@@ -48,14 +49,19 @@
 
 #include <pipewire/impl.h>
 #include <pipewire/i18n.h>
+#include <pipewire/private.h>
 
 #include <pulse/pulseaudio.h>
+#include "module-protocol-pulse/format.h"
 
 /** \page page_module_pulse_tunnel PipeWire Module: Pulse Tunnel
  */
 
 #define NAME "pulse-tunnel"
 
+PW_LOG_TOPIC_STATIC(mod_topic, "mod." NAME);
+#define PW_LOG_TOPIC_DEFAULT mod_topic
+
 #define MODULE_USAGE	"[ remote.name=<remote> ] "				\
 			"[ node.latency=<latency as fraction> ] "		\
 			"[ node.name=<name of the nodes> ] "			\
@@ -65,6 +71,7 @@
 			"[ audio.channels=<number of channels> ] "		\
 			"[ audio.position=<channel map> ] "			\
 			"pulse.server.address=<address> "			\
+			"pulse.latency=<latency in msec> "			\
 			"[ tunnel.mode=capture|playback "			\
 			"[ stream.props=<properties> ] "
 
@@ -79,7 +86,7 @@ static const struct spa_dict_item module_props[] = {
 #define RINGBUFFER_SIZE		(1u << 22)
 #define RINGBUFFER_MASK		(RINGBUFFER_SIZE-1)
 
-#define DEFAULT_LATENCY_USEC (20 * PA_USEC_PER_MSEC)
+#define DEFAULT_LATENCY_MSEC	(200)
 
 struct impl {
 	struct pw_context *context;
@@ -90,7 +97,6 @@ struct impl {
 	struct pw_properties *props;
 
 	struct pw_impl_module *module;
-	struct pw_work_queue *work;
 
 	struct spa_hook module_listener;
 
@@ -98,10 +104,11 @@ struct impl {
 	struct spa_hook core_proxy_listener;
 	struct spa_hook core_listener;
 
+	uint32_t latency_msec;
+
 	struct pw_properties *stream_props;
 	struct pw_stream *stream;
 	struct spa_hook stream_listener;
-	struct spa_io_rate_match *rate_match;
 	struct spa_audio_info_raw info;
 	uint32_t frame_size;
 
@@ -113,24 +120,13 @@ struct impl {
 	pa_context *pa_context;
 	pa_stream *pa_stream;
 
+	uint32_t target_buffer;
+	struct spa_dll dll;
+	float max_error;
+
 	unsigned int do_disconnect:1;
-	unsigned int unloading:1;
 };
 
-static void do_unload_module(void *obj, void *data, int res, uint32_t id)
-{
-	struct impl *impl = data;
-	pw_impl_module_destroy(impl->module);
-}
-
-static void unload_module(struct impl *impl)
-{
-	if (!impl->unloading) {
-		impl->unloading = true;
-		pw_work_queue_add(impl->work, impl, 0, do_unload_module, impl);
-	}
-}
-
 static void cork_stream(struct impl *impl, bool cork)
 {
 	pa_operation *operation;
@@ -167,7 +163,7 @@ static void stream_state_changed(void *d, enum pw_stream_state old,
 	switch (state) {
 	case PW_STREAM_STATE_ERROR:
 	case PW_STREAM_STATE_UNCONNECTED:
-		unload_module(impl);
+		pw_impl_module_schedule_destroy(impl->module);
 		break;
 	case PW_STREAM_STATE_PAUSED:
 		cork_stream(impl, true);
@@ -197,13 +193,29 @@ static void playback_stream_process(void *d)
 	size = SPA_MIN(bd->chunk->size, RINGBUFFER_SIZE);
 
 	filled = spa_ringbuffer_get_write_index(&impl->ring, &write_index);
+
 	if (filled < 0) {
-		pw_log_warn(NAME" %p: underrun write:%u filled:%d",
+		pw_log_warn("%p: underrun write:%u filled:%d",
 				impl, write_index, filled);
-	} else if ((uint32_t)filled + size > RINGBUFFER_SIZE) {
-		pw_log_debug(NAME" %p: overrun write:%u filled:%d size:%u max:%u",
-                                        impl, write_index, filled,
-                                        size, RINGBUFFER_SIZE);
+	} else {
+		float error, corr;
+
+		if ((uint32_t)filled + size > impl->target_buffer * 2) {
+			pw_log_warn("%p: overrun write:%u filled:%d size:%u max:%u",
+	                                        impl, write_index, filled,
+	                                        size, RINGBUFFER_SIZE);
+			write_index -= impl->target_buffer;
+			filled -= impl->target_buffer;
+		} else {
+			error = (float)filled - (float)impl->target_buffer;
+			error = SPA_CLAMP(error, -impl->max_error, impl->max_error);
+
+			pw_log_debug("filled:%u target:%u error:%f corr:%f", filled,
+					impl->target_buffer, error, corr);
+			corr = spa_dll_update(&impl->dll, error);
+			pw_stream_set_control(impl->stream,
+					SPA_PROP_rate, 1, &corr, NULL);
+		}
 	}
 	spa_ringbuffer_write_data(&impl->ring,
 				impl->buffer, RINGBUFFER_SIZE,
@@ -231,16 +243,31 @@ static void capture_stream_process(void *d)
 
 	bd = &buf->buffer->datas[0];
 
-	if (impl->rate_match)
-		req = impl->rate_match->size * impl->frame_size;
-	else
-		req = 4096;
+	if ((req = buf->requested * impl->frame_size) == 0)
+		req = 4096 * impl->frame_size;
 
 	avail = spa_ringbuffer_get_read_index(&impl->ring, &read_index);
 	if (avail <= 0) {
 		size = SPA_MIN(bd->maxsize, req);
 		memset(bd->data, 0, size);
 	} else {
+		float error, corr;
+
+		if (avail > (int32_t)impl->target_buffer * 2) {
+			avail -= impl->target_buffer;
+			read_index += impl->target_buffer;
+		} else {
+			error = (float)impl->target_buffer - (float)avail;
+			error = SPA_CLAMP(error, -impl->max_error, impl->max_error);
+
+			corr = spa_dll_update(&impl->dll, error);
+
+			pw_log_debug("avail:%u target:%u error:%f corr:%f", avail,
+					impl->target_buffer, error, corr);
+			pw_stream_set_control(impl->stream,
+					SPA_PROP_rate, 1, &corr, NULL);
+		}
+
 		size = SPA_MIN(bd->maxsize, (uint32_t)avail);
 		size = SPA_MIN(size, req);
 
@@ -251,6 +278,7 @@ static void capture_stream_process(void *d)
 
 		read_index += size;
 		spa_ringbuffer_read_update(&impl->ring, read_index);
+
 	}
 	bd->chunk->offset = 0;
 	bd->chunk->size = size;
@@ -258,16 +286,6 @@ static void capture_stream_process(void *d)
 	pw_stream_queue_buffer(impl->stream, buf);
 }
 
-static void stream_io_changed(void *data, uint32_t id, void *area, uint32_t size)
-{
-	struct impl *impl = data;
-	switch (id) {
-	case SPA_IO_RateMatch:
-		impl->rate_match = area;
-		break;
-	}
-}
-
 static const struct pw_stream_events playback_stream_events = {
 	PW_VERSION_STREAM_EVENTS,
 	.destroy = stream_destroy,
@@ -279,7 +297,6 @@ static const struct pw_stream_events capture_stream_events = {
 	PW_VERSION_STREAM_EVENTS,
 	.destroy = stream_destroy,
 	.state_changed = stream_state_changed,
-	.io_changed = stream_io_changed,
 	.process = capture_stream_process
 };
 
@@ -308,13 +325,6 @@ static int create_stream(struct impl *impl)
 				&playback_stream_events, impl);
 	}
 
-	impl->info = SPA_AUDIO_INFO_RAW_INIT(
-			.format = SPA_AUDIO_FORMAT_S16,
-			.rate = 48000,
-			.channels = 2,
-			.position = { SPA_AUDIO_CHANNEL_FL, SPA_AUDIO_CHANNEL_FR });
-	impl->frame_size = 2 * 2;
-
 	n_params = 0;
 	spa_pod_builder_init(&b, buffer, sizeof(buffer));
 	params[n_params++] = spa_format_audio_raw_build(&b,
@@ -322,7 +332,7 @@ static int create_stream(struct impl *impl)
 
 	spa_zero(latency);
 	latency.direction = impl->mode == MODE_CAPTURE ? PW_DIRECTION_OUTPUT : PW_DIRECTION_INPUT;
-	latency.min_ns = latency.max_ns = DEFAULT_LATENCY_USEC * 1000;
+	latency.min_ns = latency.max_ns = impl->latency_msec * SPA_NSEC_PER_MSEC;
 
 	params[n_params++] = spa_latency_build(&b,
 			SPA_PARAM_Latency, &latency);
@@ -380,10 +390,10 @@ static void stream_read_request_cb(pa_stream *s, size_t length, void *userdata)
 	filled = spa_ringbuffer_get_write_index(&impl->ring, &write_index);
 
 	if (filled < 0) {
-		pw_log_warn(NAME" %p: underrun write:%u filled:%d",
+		pw_log_warn("%p: underrun write:%u filled:%d",
 				impl, write_index, filled);
 	} else if (filled + length > RINGBUFFER_SIZE) {
-		pw_log_warn(NAME" %p: overrun write:%u filled:%d",
+		pw_log_warn("%p: overrun write:%u filled:%d",
 				impl, write_index, filled);
 	}
 	while (length > 0) {
@@ -485,6 +495,7 @@ static int create_pulse_stream(struct impl *impl)
 	char stream_name[1024];
 	pa_buffer_attr bufferattr;
 	int res = -EIO;
+	uint32_t latency_bytes;
 
 	if ((impl->pa_mainloop = pa_threaded_mainloop_new()) == NULL)
 		goto error;
@@ -527,9 +538,9 @@ static int create_pulse_stream(struct impl *impl)
 		pa_threaded_mainloop_wait(impl->pa_mainloop);
 	}
 
-	ss.format = PA_SAMPLE_S16NE;
-	ss.channels = 2;
-	ss.rate = 48000;
+	ss.format = (pa_sample_format_t) format_id2pa(impl->info.format);
+	ss.channels = impl->info.channels;
+	ss.rate = impl->info.rate;
 
 	snprintf(stream_name, sizeof(stream_name), _("Tunnel for %s@%s"),
 			pw_get_user_name(), pw_get_host_name());
@@ -551,19 +562,26 @@ static int create_pulse_stream(struct impl *impl)
 	bufferattr.maxlength = (uint32_t) -1;
 	bufferattr.prebuf = (uint32_t) -1;
 
+	latency_bytes = pa_usec_to_bytes(impl->latency_msec * SPA_USEC_PER_MSEC, &ss);
+
+	/* half in our buffer, half in the network + remote */
+	impl->target_buffer = latency_bytes / 2;
+
 	if (impl->mode == MODE_CAPTURE) {
-		bufferattr.fragsize = pa_usec_to_bytes(DEFAULT_LATENCY_USEC, &ss);
+		bufferattr.fragsize = latency_bytes / 2;
 
 		res = pa_stream_connect_record(impl->pa_stream,
 				remote_node_target, &bufferattr,
+				PA_STREAM_DONT_MOVE |
 				PA_STREAM_INTERPOLATE_TIMING |
 				PA_STREAM_ADJUST_LATENCY |
 				PA_STREAM_AUTO_TIMING_UPDATE);
 	} else {
-		bufferattr.tlength = pa_usec_to_bytes(DEFAULT_LATENCY_USEC, &ss);
+		bufferattr.tlength = latency_bytes / 2;
 
 		res = pa_stream_connect_playback(impl->pa_stream,
 				remote_node_target, &bufferattr,
+				PA_STREAM_DONT_MOVE |
 				PA_STREAM_INTERPOLATE_TIMING |
 				PA_STREAM_ADJUST_LATENCY |
 				PA_STREAM_AUTO_TIMING_UPDATE,
@@ -611,7 +629,7 @@ static void core_error(void *data, uint32_t id, int seq, int res, const char *me
 			id, seq, res, spa_strerror(res), message);
 
 	if (id == PW_ID_CORE && res == -EPIPE)
-		unload_module(impl);
+		pw_impl_module_schedule_destroy(impl->module);
 }
 
 static const struct pw_core_events core_events = {
@@ -624,7 +642,7 @@ static void core_destroy(void *d)
 	struct impl *impl = d;
 	spa_hook_remove(&impl->core_listener);
 	impl->core = NULL;
-	unload_module(impl);
+	pw_impl_module_schedule_destroy(impl->module);
 }
 
 static const struct pw_proxy_events core_proxy_events = {
@@ -653,8 +671,6 @@ static void impl_destroy(struct impl *impl)
 	pw_properties_free(impl->stream_props);
 	pw_properties_free(impl->props);
 
-	if (impl->work)
-		pw_work_queue_cancel(impl->work, impl, SPA_ID_INVALID);
 	free(impl->buffer);
 	free(impl);
 }
@@ -662,7 +678,6 @@ static void impl_destroy(struct impl *impl)
 static void module_destroy(void *data)
 {
 	struct impl *impl = data;
-	impl->unloading = true;
 	spa_hook_remove(&impl->module_listener);
 	impl_destroy(impl);
 }
@@ -698,18 +713,72 @@ static void parse_position(struct spa_audio_info_raw *info, const char *val, siz
 	}
 }
 
+static inline uint32_t format_from_name(const char *name, size_t len)
+{
+	int i;
+	for (i = 0; spa_type_audio_format[i].name; i++) {
+		if (strncmp(name, spa_debug_type_short_name(spa_type_audio_format[i].name), len) == 0)
+			return spa_type_audio_format[i].type;
+	}
+	return SPA_AUDIO_FORMAT_UNKNOWN;
+}
+
 static void parse_audio_info(struct pw_properties *props, struct spa_audio_info_raw *info)
 {
 	const char *str;
 
 	*info = SPA_AUDIO_INFO_RAW_INIT(
-			.format = SPA_AUDIO_FORMAT_F32P);
-	if ((str = pw_properties_get(props, PW_KEY_AUDIO_RATE)) != NULL)
-		info->rate = atoi(str);
-	if ((str = pw_properties_get(props, PW_KEY_AUDIO_CHANNELS)) != NULL)
-		info->channels = atoi(str);
+			.rate = 48000,
+			.channels = 2,
+			.format = SPA_AUDIO_FORMAT_S16);
+
+	if ((str = pw_properties_get(props, PW_KEY_AUDIO_FORMAT)) != NULL) {
+		uint32_t id;
+
+		id = format_from_name(str, strlen(str));
+		if (id != SPA_AUDIO_FORMAT_UNKNOWN)
+			info->format = id;
+	}
+
+	info->rate = pw_properties_get_uint32(props, PW_KEY_AUDIO_RATE, info->rate);
+	info->channels = pw_properties_get_uint32(props, PW_KEY_AUDIO_CHANNELS, info->channels);
 	if ((str = pw_properties_get(props, SPA_KEY_AUDIO_POSITION)) != NULL)
 		parse_position(info, str, strlen(str));
+
+}
+
+static int calc_frame_size(struct spa_audio_info_raw *info)
+{
+	int res = info->channels;
+	switch (info->format) {
+	case SPA_AUDIO_FORMAT_U8:
+	case SPA_AUDIO_FORMAT_S8:
+	case SPA_AUDIO_FORMAT_ALAW:
+	case SPA_AUDIO_FORMAT_ULAW:
+		return res;
+	case SPA_AUDIO_FORMAT_S16:
+	case SPA_AUDIO_FORMAT_S16_OE:
+	case SPA_AUDIO_FORMAT_U16:
+		return res * 2;
+	case SPA_AUDIO_FORMAT_S24:
+	case SPA_AUDIO_FORMAT_S24_OE:
+	case SPA_AUDIO_FORMAT_U24:
+		return res * 3;
+	case SPA_AUDIO_FORMAT_S24_32:
+	case SPA_AUDIO_FORMAT_S24_32_OE:
+	case SPA_AUDIO_FORMAT_S32:
+	case SPA_AUDIO_FORMAT_S32_OE:
+	case SPA_AUDIO_FORMAT_U32:
+	case SPA_AUDIO_FORMAT_U32_OE:
+	case SPA_AUDIO_FORMAT_F32:
+	case SPA_AUDIO_FORMAT_F32_OE:
+		return res * 4;
+	case SPA_AUDIO_FORMAT_F64:
+	case SPA_AUDIO_FORMAT_F64_OE:
+		return res * 8;
+	default:
+		return 0;
+	}
 }
 
 static void copy_props(struct impl *impl, struct pw_properties *props, const char *key)
@@ -730,6 +799,8 @@ int pipewire__module_init(struct pw_impl_module *module, const char *args)
 	const char *str;
 	int res;
 
+	PW_LOG_TOPIC_INIT(mod_topic);
+
 	impl = calloc(1, sizeof(struct impl));
 	if (impl == NULL)
 		return -errno;
@@ -756,15 +827,10 @@ int pipewire__module_init(struct pw_impl_module *module, const char *args)
 
 	impl->module = module;
 	impl->context = context;
-	impl->work = pw_context_get_work_queue(context);
-	if (impl->work == NULL) {
-		res = -errno;
-		pw_log_error( "can't get work queue: %m");
-		goto error;
-	}
 
 	spa_ringbuffer_init(&impl->ring);
 	impl->buffer = calloc(1, RINGBUFFER_SIZE);
+	spa_dll_init(&impl->dll);
 
 	if ((str = pw_properties_get(props, "tunnel.mode")) != NULL) {
 		if (spa_streq(str, "capture")) {
@@ -777,10 +843,16 @@ int pipewire__module_init(struct pw_impl_module *module, const char *args)
 			goto error;
 		}
 	}
-	if (pw_properties_get(props, PW_KEY_NODE_GROUP) == NULL)
-		pw_properties_set(props, PW_KEY_NODE_GROUP, "pipewire.dummy");
+
+	impl->latency_msec = pw_properties_get_uint32(props, "pulse.latency", DEFAULT_LATENCY_MSEC);
+
+
+	if (pw_properties_get(props, PW_KEY_NODE_WANT_DRIVER) == NULL)
+		pw_properties_set(props, PW_KEY_NODE_WANT_DRIVER, "true");
 	if (pw_properties_get(props, PW_KEY_NODE_VIRTUAL) == NULL)
 		pw_properties_set(props, PW_KEY_NODE_VIRTUAL, "true");
+	if (pw_properties_get(props, PW_KEY_NODE_NETWORK) == NULL)
+		pw_properties_set(props, PW_KEY_NODE_NETWORK, "true");
 
 	if (pw_properties_get(props, PW_KEY_MEDIA_CLASS) == NULL)
 		pw_properties_set(props, PW_KEY_MEDIA_CLASS,
@@ -790,18 +862,30 @@ int pipewire__module_init(struct pw_impl_module *module, const char *args)
 	if ((str = pw_properties_get(props, "stream.props")) != NULL)
 		pw_properties_update_string(impl->stream_props, str, strlen(str));
 
+	copy_props(impl, props, PW_KEY_AUDIO_FORMAT);
 	copy_props(impl, props, PW_KEY_AUDIO_RATE);
 	copy_props(impl, props, PW_KEY_AUDIO_CHANNELS);
 	copy_props(impl, props, SPA_KEY_AUDIO_POSITION);
 	copy_props(impl, props, PW_KEY_NODE_NAME);
 	copy_props(impl, props, PW_KEY_NODE_DESCRIPTION);
 	copy_props(impl, props, PW_KEY_NODE_GROUP);
+	copy_props(impl, props, PW_KEY_NODE_WANT_DRIVER);
 	copy_props(impl, props, PW_KEY_NODE_LATENCY);
 	copy_props(impl, props, PW_KEY_NODE_VIRTUAL);
 	copy_props(impl, props, PW_KEY_MEDIA_CLASS);
 
 	parse_audio_info(impl->stream_props, &impl->info);
 
+	impl->frame_size = calc_frame_size(&impl->info);
+	if (impl->frame_size == 0) {
+		pw_log_error("unsupported audio format:%d channels:%d",
+				impl->info.format, impl->info.channels);
+		res = -EINVAL;
+		goto error;
+	}
+	spa_dll_set_bw(&impl->dll, SPA_DLL_BW_MIN, 128, impl->info.rate);
+	impl->max_error = 256.0f * impl->frame_size;
+
 	impl->core = pw_context_get_object(impl->context, PW_TYPE_INTERFACE_Core);
 	if (impl->core == NULL) {
 		str = pw_properties_get(props, PW_KEY_REMOTE_NAME);
