@@ -88,6 +88,11 @@
  *
  */
 
+#define NAME "roc-source"
+
+PW_LOG_TOPIC_STATIC(mod_topic, "mod." NAME);
+#define PW_LOG_TOPIC_DEFAULT mod_topic
+
 #define ROC_DEFAULT_IP           "0.0.0.0"
 #define ROC_DEFAULT_SOURCE_PORT  10001
 #define ROC_DEFAULT_REPAIR_PORT  10002
@@ -95,7 +100,6 @@
 
 struct module_roc_source_data {
 	struct pw_impl_module *module;
-	struct pw_work_queue *work;
 	struct spa_hook module_listener;
 	struct pw_properties *props;
 	struct pw_context *module_context;
@@ -109,7 +113,6 @@ struct module_roc_source_data {
 	struct pw_properties *playback_props;
 
 	unsigned int do_disconnect:1;
-	unsigned int unloading:1;
 
 	roc_address local_addr;
 	roc_address local_source_addr;
@@ -124,20 +127,6 @@ struct module_roc_source_data {
 	int sess_latency_msec;
 };
 
-static void do_unload_module(void *obj, void *d, int res, uint32_t id)
-{
-	struct module_roc_source_data *data = d;
-	pw_impl_module_destroy(data->module);
-}
-
-static void unload_module(struct module_roc_source_data *data)
-{
-	if (!data->unloading) {
-		data->unloading = true;
-		pw_work_queue_add(data->work, data, 0, do_unload_module, data);
-	}
-}
-
 static void stream_destroy(void *d)
 {
 	struct module_roc_source_data *data = d;
@@ -197,7 +186,7 @@ static void playback_process(void *data)
 	if (roc_receiver_read(impl->receiver, &frame) != 0) {
 		/* Handle EOF and error */
 		pw_log_error("Failed to read from roc source");
-		unload_module(data);
+		pw_impl_module_schedule_destroy(impl->module);
 		return;
 	}
 
@@ -214,7 +203,7 @@ static void on_core_error(void *d, uint32_t id, int seq, int res, const char *me
 			id, seq, res, spa_strerror(res), message);
 
 	if (id == PW_ID_CORE && res == -EPIPE)
-		unload_module(data);
+		pw_impl_module_schedule_destroy(data->module);
 }
 
 static const struct pw_core_events core_events = {
@@ -230,7 +219,7 @@ static void on_stream_state_changed(void *d, enum pw_stream_state old,
 	switch (state) {
 	case PW_STREAM_STATE_UNCONNECTED:
 		pw_log_info("stream disconnected, unloading");
-		unload_module(data);
+		pw_impl_module_schedule_destroy(data->module);
 		break;
 	case PW_STREAM_STATE_ERROR:
 		pw_log_error("stream error: %s", error);
@@ -252,7 +241,7 @@ static void core_destroy(void *d)
 	struct module_roc_source_data *data = d;
 	spa_hook_remove(&data->core_listener);
 	data->core = NULL;
-	unload_module(data);
+	pw_impl_module_schedule_destroy(data->module);
 }
 
 static const struct pw_proxy_events core_proxy_events = {
@@ -269,8 +258,6 @@ static void impl_destroy(struct module_roc_source_data *data)
 	pw_properties_free(data->playback_props);
 	pw_properties_free(data->props);
 
-	if (data->work)
-		pw_work_queue_cancel(data->work, data, SPA_ID_INVALID);
 	if (data->receiver)
 		roc_receiver_close(data->receiver);
 	if (data->context)
@@ -284,7 +271,6 @@ static void impl_destroy(struct module_roc_source_data *data)
 static void module_destroy(void *d)
 {
 	struct module_roc_source_data *data = d;
-	data->unloading = true;
 	spa_hook_remove(&data->module_listener);
 	impl_destroy(data);
 }
@@ -429,6 +415,8 @@ int pipewire__module_init(struct pw_impl_module *module, const char *args)
 	char *local_ip = NULL, *resampler_profile = NULL;
 	int res = 0, local_repair_port, local_source_port, sess_latency_msec;
 
+	PW_LOG_TOPIC_INIT(mod_topic);
+
 	data = calloc(1, sizeof(struct module_roc_source_data));
 	if (data == NULL)
 		return -errno;
@@ -454,12 +442,6 @@ int pipewire__module_init(struct pw_impl_module *module, const char *args)
 
 	data->module = module;
 	data->module_context = context;
-	data->work = pw_context_get_work_queue(context);
-	if (data->work == NULL) {
-		res = -errno;
-		pw_log_error( "can't get work queue: %m");
-		goto out;
-	}
 
 	if ((str = pw_properties_get(props, "source.name")) != NULL) {
 		pw_properties_set(playback_props, PW_KEY_NODE_NAME, str);
@@ -473,12 +455,12 @@ int pipewire__module_init(struct pw_impl_module *module, const char *args)
 		pw_properties_set(playback_props, PW_KEY_NODE_NAME, "roc-source");
 	if (pw_properties_get(playback_props, PW_KEY_NODE_DESCRIPTION) == NULL)
 		pw_properties_set(playback_props, PW_KEY_NODE_DESCRIPTION, "ROC Source");
-	if (pw_properties_get(playback_props, PW_KEY_NODE_GROUP) == NULL)
-		pw_properties_set(playback_props, PW_KEY_NODE_GROUP, "pipewire.dummy");
+	if (pw_properties_get(playback_props, PW_KEY_NODE_WANT_DRIVER) == NULL)
+		pw_properties_set(playback_props, PW_KEY_NODE_WANT_DRIVER, "true");
 	if (pw_properties_get(playback_props, PW_KEY_NODE_VIRTUAL) == NULL)
 		pw_properties_set(playback_props, PW_KEY_NODE_VIRTUAL, "true");
-	if (pw_properties_get(playback_props, PW_KEY_NODE_PASSIVE) == NULL)
-		pw_properties_set(playback_props, PW_KEY_NODE_PASSIVE, "true");
+	if (pw_properties_get(playback_props, PW_KEY_NODE_NETWORK) == NULL)
+		pw_properties_set(playback_props, PW_KEY_NODE_NETWORK, "true");
 	if ((str = pw_properties_get(playback_props, PW_KEY_MEDIA_CLASS)) == NULL)
 		pw_properties_set(playback_props, PW_KEY_MEDIA_CLASS, "Audio/Source");
 
