@@ -24,8 +24,8 @@
  */
 
 #include <pipewire/impl.h>
-#include <extensions/session-manager.h>
-#include <extensions/session-manager/introspect-funcs.h>
+#include <pipewire/extensions/session-manager.h>
+#include <pipewire/extensions/session-manager/introspect-funcs.h>
 
 #include <spa/utils/result.h>
 #include <spa/pod/builder.h>
@@ -221,6 +221,7 @@ static void global_destroy(void *data)
 	impl->global = NULL;
 	if (impl->resource)
 		pw_resource_destroy(impl->resource);
+	free(impl);
 }
 
 static const struct pw_global_events global_events = {
@@ -234,6 +235,7 @@ static void impl_resource_destroy(void *data)
 	struct param_data *pdata, *tmp;
 
 	spa_hook_remove(&impl->resource_listener);
+	spa_hook_remove(&impl->link_listener);
 	impl->resource = NULL;
 
 	/* clear cache */
@@ -387,6 +389,15 @@ static void *link_new(struct pw_context *context,
 			  struct pw_properties *properties)
 {
 	struct impl *impl;
+	char serial_str[32];
+	struct spa_dict_item items[1] = {
+		SPA_DICT_ITEM_INIT(PW_KEY_OBJECT_SERIAL, serial_str),
+	};
+	struct spa_dict extra_props = SPA_DICT_INIT_ARRAY(items);
+	static const char * const keys[] = {
+		PW_KEY_OBJECT_SERIAL,
+		NULL
+	};
 
 	impl = calloc(1, sizeof(*impl));
 	if (impl == NULL) {
@@ -405,6 +416,10 @@ static void *link_new(struct pw_context *context,
 	}
 	impl->resource = resource;
 
+	spa_scnprintf(serial_str, sizeof(serial_str), "%"PRIu64,
+			pw_global_get_serial(impl->global));
+	pw_global_update_keys(impl->global, &extra_props, keys);
+
 	spa_list_init(&impl->cached_params);
 
 	/* handle destroy events */
@@ -525,6 +540,7 @@ int endpoint_link_factory_init(struct pw_impl_module *module)
 	struct pw_context *context = pw_impl_module_get_context(module);
 	struct pw_impl_factory *factory;
 	struct factory_data *data;
+	int res;
 
 	factory = pw_context_create_factory(context,
 				 "endpoint-link",
@@ -543,9 +559,13 @@ int endpoint_link_factory_init(struct pw_impl_module *module)
 
 	data->export.type = PW_TYPE_INTERFACE_EndpointLink;
 	data->export.func = pw_core_endpoint_link_export;
-	pw_context_register_export_type(context, &data->export);
+	if ((res = pw_context_register_export_type(context, &data->export)) < 0)
+		goto error;
 
 	pw_impl_module_add_listener(module, &data->module_listener, &module_events, data);
 
 	return 0;
+error:
+	pw_impl_factory_destroy(data->this);
+	return res;
 }
