@@ -51,7 +51,7 @@ struct ucred {
 
 #define MAX_RATES				16u
 #define CLOCK_MIN_QUANTUM			4u
-#define CLOCK_MAX_QUANTUM			8192u
+#define CLOCK_MAX_QUANTUM			65536u
 
 struct settings {
 	uint32_t log_level;
@@ -61,12 +61,15 @@ struct settings {
 	uint32_t clock_quantum;			/* default quantum */
 	uint32_t clock_min_quantum;		/* min quantum */
 	uint32_t clock_max_quantum;		/* max quantum */
+	uint32_t clock_quantum_limit;		/* quantum limit */
 	struct spa_rectangle video_size;
 	struct spa_fraction video_rate;
 	uint32_t link_max_buffers;
 	unsigned int mem_warn_mlock:1;
 	unsigned int mem_allow_mlock:1;
 	unsigned int clock_power_of_two_quantum:1;
+	unsigned int check_quantum:1;
+	unsigned int check_rate:1;
 #define CLOCK_RATE_UPDATE_MODE_HARD 0
 #define CLOCK_RATE_UPDATE_MODE_SOFT 1
 	int clock_rate_update_mode;
@@ -296,6 +299,8 @@ struct pw_impl_client {
 	struct pw_protocol *protocol;	/**< protocol in use */
 	int recv_seq;			/**< last received sequence number */
 	int send_seq;			/**< last sender sequence number */
+	uint64_t recv_generation;	/**< last received registry generation */
+	uint64_t sent_generation;	/**< last sent registry generation */
 
 	void *user_data;		/**< extra user data */
 
@@ -331,6 +336,7 @@ struct pw_global {
 	pw_global_bind_func_t func;	/**< bind function */
 	void *object;			/**< object associated with the interface */
 	uint64_t serial;		/**< increasing serial number */
+	uint64_t generation;		/**< registry generation number */
 
 	struct spa_list resource_list;	/**< The list of resources of this global */
 
@@ -357,7 +363,11 @@ pw_core_resource_errorv(struct pw_resource *resource, uint32_t id, int seq,
 	buffer[1023] = '\0';
 	pw_log_debug("resource %p: id:%d seq:%d res:%d (%s) msg:\"%s\"",
 			resource, id, seq, res, spa_strerror(res), buffer);
-	pw_core_resource_error(resource, id, seq, res, buffer);
+	if (resource)
+		pw_core_resource_error(resource, id, seq, res, buffer);
+	else
+		pw_log_error("id:%d seq:%d res:%d (%s) msg:\"%s\"",
+				id, seq, res, spa_strerror(res), buffer);
 }
 
 static inline SPA_PRINTF_FUNC(5,6) void
@@ -421,6 +431,9 @@ struct pw_context {
 
 	struct pw_mempool *pool;		/**< global memory pool */
 
+	uint64_t stamp;
+	uint64_t serial;
+	uint64_t generation;			/**< registry generation number */
 	struct pw_map globals;			/**< map of globals */
 
 	struct spa_list core_impl_list;		/**< list of core_imp */
@@ -442,6 +455,7 @@ struct pw_context {
 	struct spa_hook_list driver_listener_list;
 	struct spa_hook_list listener_list;
 
+	struct spa_thread_utils *thread_utils;
 	struct pw_loop *main_loop;		/**< main loop for control */
 	struct pw_loop *data_loop;		/**< data loop for data passing */
 	struct pw_data_loop *data_loop_impl;
@@ -469,9 +483,11 @@ struct pw_data_loop {
 	struct pw_loop *loop;
 
 	struct spa_hook_list listener_list;
-	struct spa_source *event;
+
+	struct spa_thread_utils *thread_utils;
 
 	pthread_t thread;
+	unsigned int cancel:1;
 	unsigned int created:1;
 	unsigned int running:1;
 };
@@ -483,7 +499,6 @@ struct pw_main_loop {
         struct pw_loop *loop;
 
 	struct spa_hook_list listener_list;
-	struct spa_source *event;
 
 	unsigned int created:1;
 	unsigned int running:1;
@@ -684,6 +699,7 @@ struct pw_impl_node {
 	unsigned int lock_quantum:1;	/**< don't change graph quantum */
 	unsigned int lock_rate:1;	/**< don't change graph rate */
 	unsigned int transport_sync:1;	/**< supports transport sync */
+	unsigned int current_pending:1;	/**< a quantum/rate update is pending */
 
 	uint32_t port_user_data_size;	/**< extra size for port user data */
 
@@ -709,6 +725,9 @@ struct pw_impl_node {
 	struct spa_fraction latency;		/**< requested latency */
 	struct spa_fraction max_latency;	/**< maximum latency */
 	struct spa_fraction rate;		/**< requested rate */
+	uint32_t force_quantum;			/**< forced quantum */
+	uint32_t force_rate;			/**< forced rate */
+	uint32_t stamp;				/**< stamp of last update */
 	struct spa_source source;		/**< source to remotely trigger this node */
 	struct pw_memblock *activation;
 	struct {
@@ -989,6 +1008,7 @@ struct pw_core {
 	struct pw_protocol_client *conn;	/**< the protocol client connection */
 	int recv_seq;				/**< last received sequence number */
 	int send_seq;				/**< last protocol result code */
+	uint64_t recv_generation;		/**< last received registry generation */
 
 	unsigned int removed:1;
 	unsigned int destroyed:1;
@@ -1267,6 +1287,8 @@ void pw_settings_init(struct pw_context *context);
 int pw_settings_expose(struct pw_context *context);
 void pw_settings_clean(struct pw_context *context);
 
+void pw_impl_module_schedule_destroy(struct pw_impl_module *module);
+
 /** \endcond */
 
 #ifdef __cplusplus
